// LICENSE: MIT
"use strict";
// imports
Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _awsSdk = require('aws-sdk');

var _awsSdk2 = _interopRequireDefault(_awsSdk);

var _baconjs = require('baconjs');

var _baconjs2 = _interopRequireDefault(_baconjs);

var _schedulerJs = require('./scheduler.js');

var _schedulerJs2 = _interopRequireDefault(_schedulerJs);

var _entitiesMessageJs = require('./entities/message.js');

var _entitiesMessageJs2 = _interopRequireDefault(_entitiesMessageJs);

/**
 * CloudWatch logs of describe in the stream.
 */

var LambdaLogsStream = (function () {
  /**
   * Initialize LambdaLogsStream.
   */

  function LambdaLogsStream() {
    _classCallCheck(this, LambdaLogsStream);

    this.logs = _bluebird2['default'].promisifyAll(new _awsSdk2['default'].CloudWatchLogs());
  }

  /**
   * CloudWatch logs of describe in the stream.
   *
   * @param {String} functionName function name of AWS Lambda.
   * @param {Number} [startTime=null] to get the log after the specified time
   * @returns {Bacon.Bus} return a stream of LambdaLogs
   */

  _createClass(LambdaLogsStream, [{
    key: 'describe',
    value: function describe(functionName) {
      var startTime = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

      // initialize
      var logs = this.logs;
      var logGroupName = '/aws/lambda/' + functionName;
      // generate scheduler
      var scheduler = new _schedulerJs2['default'](1000, function (index) {
        var params = {
          logGroupName: logGroupName
        };
        return logs.describeLogStreamsAsync(params);
      });
      // get log event
      var stream = scheduler.start().flatMap(_baconjs2['default'].fromPromise).map(function (data) {
        return data.logStreams.map(function (logStream) {
          var params = {
            logGroupName: logGroupName,
            logStreamName: logStream.logStreamName,
            startTime: startTime
          };
          return logs.getLogEventsAsync(params);
        });
      }).flatMap(_baconjs2['default'].fromArray).flatMap(_baconjs2['default'].fromPromise)
      // log event to event
      .map(function (logEvent) {
        return logEvent.events;
      }).flatMap(_baconjs2['default'].fromArray)
      // event to message
      .map(function (event) {
        if (startTime < event.timestamp) {
          startTime = event.timestamp + 1;
        }
        return event.message.replace(/[\n\r\t]+$/, '');;
      })
      // message to message entity
      .map(function (message) {
        return new _entitiesMessageJs2['default'](message);
      });
      // return stream
      stream.stop = function () {
        scheduler.stop();
      };
      return stream;
    }
  }]);

  return LambdaLogsStream;
})();

exports['default'] = LambdaLogsStream;
module.exports = exports['default'];